/***********************************************************
 * $Id: OdtContentWriter.java 13 2007-12-17 19:32:15Z lech $
 * 
 * OpenDocument transformation tool of the clazzes.org project
 * http://www.clazzes.org
 *
 * Copyright (C) 2006-2007 ev-i Informationstechnologie GmbH
 *
 * Created: 2006-12-18
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.odtransform;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * This class is used to serialize XML content to an OpenDocument
 * output file. The SAX events are assumed to conform with a full-featutred
 * OpenDocument file comprised of an <tt>office:document</tt> root tag.
 * 
 * @author wglas
 */
public class OdtContentWriter implements ContentHandler
{
    private static Log log = LogFactory.getLog(OdtContentWriter.class);
    
    private ContentHandler outputHandlers[];
    private ZipOutputStream zos;
    private ByteArrayOutputStream metaOS;
    private ByteArrayOutputStream settingsOS;
    private ByteArrayOutputStream stylesOS;
    private ZipFile assetFile;
    
    /**
     * Cretate a new Odt writer for use as a SAX {@link ContentHandler} in XML transformations.
     * 
     * @param assetFile All entries from this ZIP file exept meta.xml, settings.xml, content.xml and styles.xml
     *              are imported to the new ODT file.
     * @param zos The output stream to which the content is serialized. Please note, that you have to
     *              close the zip output stream at your own. Otherwise, the written zip file will be incomplete.
     * @throws TransformerConfigurationException
     * @throws IOException
     */
    public OdtContentWriter(ZipFile assetFile, ZipOutputStream zos) throws TransformerConfigurationException, IOException
    {
        this.assetFile = assetFile;
        this.zos = zos;
        
        TransformerFactory transformerFactory = SAXTransformerFactory.newInstance();
        
        if (!(transformerFactory instanceof SAXTransformerFactory))
            throw new TransformerConfigurationException("TransformerFactory.newInstance() did not return an instance of javax.xml.transform.sax.SAXTransformerFactory.");
        
        SAXTransformerFactory saxTransformerFactory =
            (SAXTransformerFactory) transformerFactory;

        ClassLoader loader = OdtContentWriter.class.getClassLoader();
        
        // meta.xml output handler.
        log.debug("Setting up meta.xml output handler.");
        InputStream odtMetaIS =
            loader.getResourceAsStream("org/clazzes/odtransform/odtmeta.xslt");

        TransformerHandler odtMetaHandler =
            saxTransformerFactory.newTransformerHandler(new StreamSource(odtMetaIS));
        
        this.metaOS = new ByteArrayOutputStream();
        
        odtMetaHandler.setResult(new StreamResult(this.metaOS));
        
        // settings.xml output handler.
        log.debug("Setting up settings.xml output handler.");
        InputStream odtSettingsIS =
           loader.getResourceAsStream("org/clazzes/odtransform/odtsettings.xslt");
        
        TransformerHandler odtSettingsHandler =
            saxTransformerFactory.newTransformerHandler(new StreamSource(odtSettingsIS));

        this.settingsOS = new ByteArrayOutputStream();
        
        odtSettingsHandler.setResult(new StreamResult(this.settingsOS));

        // content.xml output handler.
        log.debug("Setting up content.xml output handler.");
        InputStream odtContentIS =
            loader.getResourceAsStream("org/clazzes/odtransform/odtcontent.xslt");
        
        TransformerHandler odtContentHandler =
            saxTransformerFactory.newTransformerHandler(new StreamSource(odtContentIS));

        this.zos.putNextEntry(new ZipEntry("content.xml"));
        
        odtContentHandler.setResult(new StreamResult(this.zos));
        
        // styles.xml output handler.
        log.debug("Setting up styles.xml output handler.");
                InputStream odtStylesIS =
            loader.getResourceAsStream("org/clazzes/odtransform/odtstyles.xslt");

        TransformerHandler odtStylesHandler =
            saxTransformerFactory.newTransformerHandler(new StreamSource(odtStylesIS));

        this.stylesOS = new ByteArrayOutputStream();
        
        odtStylesHandler.setResult(new StreamResult(this.stylesOS));

        // generate content handler, which multiplexes to the four partial xml files.
        this.outputHandlers = new ContentHandler[]
                { odtMetaHandler, odtSettingsHandler, odtContentHandler, odtStylesHandler };
    }
    
    
    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char[] arg0, int arg1, int arg2) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.characters(arg0, arg1, arg2);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endDocument()
     */
    public void endDocument() throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.endDocument();
        
        try
        {
            log.debug("Closing content.xml zip entry.");
            this.zos.closeEntry();
            
            log.debug("Writing meta.xml zip entry.");
            this.zos.putNextEntry(new ZipEntry("meta.xml"));
            this.zos.write(this.metaOS.toByteArray());
            this.zos.closeEntry();

            log.debug("Writing settings.xml zip entry.");
            this.zos.putNextEntry(new ZipEntry("settings.xml"));
            this.zos.write(this.settingsOS.toByteArray());
            this.zos.closeEntry();
            
            log.debug("Writing styles.xml zip entry.");
            this.zos.putNextEntry(new ZipEntry("styles.xml"));
            this.zos.write(this.stylesOS.toByteArray());
            this.zos.closeEntry();
            
            if (this.assetFile == null) return;
            
            Enumeration<? extends ZipEntry> assetEntries = this.assetFile.entries();
            
            while(assetEntries.hasMoreElements())
            {
                ZipEntry e = assetEntries.nextElement();
                
                if (e.isDirectory()) continue;
                
                if ("meta.xml".equals(e.getName()) ||
                        "settings.xml".equals(e.getName()) ||
                        "content.xml".equals(e.getName()) ||
                        "styles.xml".equals(e.getName())   )
                    continue;
                            
                log.debug("Writing asset entry ["+e.getName()+"].");

                InputStream is = this.assetFile.getInputStream(e);
                this.zos.putNextEntry(new ZipEntry(e.getName()));
                
                byte[] buf=new byte[16384];
                int n;
                
                while ((n=is.read(buf)) > 0)
                {
                    this.zos.write(buf,0,n);
                }
                
                is.close();
                this.zos.closeEntry();
            }
           
        } catch (IOException e)
        {
            throw new SAXException("Unexpected I/O error pupulating ZIP output stream",e);
        }
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    public void endElement(String arg0, String arg1, String arg2) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.endElement(arg0, arg1, arg2);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#endPrefixMapping(java.lang.String)
     */
    public void endPrefixMapping(String arg0) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.endPrefixMapping(arg0);   
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int)
     */
    public void ignorableWhitespace(char[] arg0, int arg1, int arg2) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.ignorableWhitespace(arg0, arg1, arg2);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String)
     */
    public void processingInstruction(String arg0, String arg1) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.processingInstruction(arg0, arg1);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#setDocumentLocator(org.xml.sax.Locator)
     */
    public void setDocumentLocator(Locator arg0)
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.setDocumentLocator(arg0);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#skippedEntity(java.lang.String)
     */
    public void skippedEntity(String arg0) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.skippedEntity(arg0);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startDocument()
     */
    public void startDocument() throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.startDocument();
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String arg0, String arg1, String arg2, Attributes arg3) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.startElement(arg0, arg1, arg2, arg3);
    }

    /* (non-Javadoc)
     * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)
     */
    public void startPrefixMapping(String arg0, String arg1) throws SAXException
    {
        for (ContentHandler outputHandler : this.outputHandlers)
            outputHandler.startPrefixMapping(arg0, arg1);
    }
}
