/***********************************************************
 * $Id: TestTransform.java 13 2007-12-17 19:32:15Z lech $
 * 
 * OpenDocument transformation tool of the clazzes.org project
 * http://www.clazzes.org
 *
 * Copyright (C) 2006-2007 ev-i Informationstechnologie GmbH
 *
 * Created: Dec 18, 2006
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.odtransform;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TransformerHandler;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * @author wglas
 *
 */
public class TestTransform extends TestCase
{
    private File outputDir;
    
    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception
    {
        this.outputDir = new File("target/test");
        
        if (!this.outputDir.isDirectory())
            if (!this.outputDir.mkdirs())
                throw new IOException("Cannot create output directory ["+this.outputDir+"].");
    }

    private void assertFileEquals(File a, File b) throws IOException
    {
        if (a.length() != b.length())
            throw new AssertionFailedError("length ["+a.length()+ "] of file ["+a+"] differs from length ["+b.length()+ "] of file ["+b+"].");
        
        FileInputStream ais = new FileInputStream(a);
        FileInputStream bis = new FileInputStream(b);
        
        int ca;
        long pos = 0;
        
        while ((ca=ais.read()) >= 0)
        {
            int cb = bis.read();
            
            if (ca!=cb)
                throw new AssertionFailedError("byte ["+ca+ "] of file ["+a+"] differs from byte ["+cb+ "] of file ["+b+"] at file position ["+pos+"].");
            
            ++pos;
        }
        
    }
    
    private void testMergeInternal(String srcPath, String destFile) throws IOException, TransformerConfigurationException, SAXException
    {
        ZipFile odtFile = new ZipFile(srcPath);
        
        OdtTransformer transformer = new OdtTransformer();
        
        File res = new File(this.outputDir,destFile);
        
        SAXTransformerFactory transformerFactory =
            (SAXTransformerFactory) SAXTransformerFactory.newInstance();
        
        StreamSource sortSource =
            new StreamSource(TestTransform.class.getClassLoader().getResourceAsStream("org/clazzes/odtransform/odtsortstyles.xslt"));
            
        TransformerHandler transformerHandler = transformerFactory.newTransformerHandler(sortSource);
  
        StreamResult result = new StreamResult(res);

        transformerHandler.setResult(result);
        
        transformer.transform(odtFile, new SAXResult(transformerHandler));
        
        assertFileEquals(new File("src/test/resources/com/galactic/odt/FeeTransaction.xml"), res);
    }
    
    public void testMerge() throws IOException, TransformerConfigurationException, SAXException
    {
        this.testMergeInternal("src/test/resources/com/galactic/odt/FeeTransaction.odt", "FeeTransaction.xml");
    }

    
    public void testMetaXslt() throws TransformerException
    {
        File src = new File("src/test/resources/com/galactic/odt/FeeTransaction.xml");
        File res = new File(this.outputDir,"meta.xml");
       
        InputStream metaXsltIS =
            TestTransform.class.getClassLoader().getResourceAsStream("org/clazzes/odtransform/odtmeta.xslt");
        
        TransformerFactory transformerFactory =
            TransformerFactory.newInstance();
        
        Transformer transformer =
            transformerFactory.newTransformer(new StreamSource(metaXsltIS));
        
        transformer.transform(new StreamSource(src), new StreamResult(res));
     }
    
    public void testSettingsXslt() throws TransformerException
    {
        File src = new File("src/test/resources/com/galactic/odt/FeeTransaction.xml");
        File res = new File(this.outputDir,"settings.xml");
       
        InputStream settingsXsltIS =
            TestTransform.class.getClassLoader().getResourceAsStream("org/clazzes/odtransform/odtsettings.xslt");
        
        TransformerFactory transformerFactory =
            TransformerFactory.newInstance();
        
        Transformer transformer =
            transformerFactory.newTransformer(new StreamSource(settingsXsltIS));
        
        transformer.transform(new StreamSource(src), new StreamResult(res));
     }
    
    public void testStylesXslt() throws TransformerException
    {
        File src = new File("src/test/resources/com/galactic/odt/FeeTransaction.xml");
        File res = new File(this.outputDir,"styles.xml");
       
        InputStream stylesXsltIS =
            TestTransform.class.getClassLoader().getResourceAsStream("org/clazzes/odtransform/odtstyles.xslt");
        
        TransformerFactory transformerFactory =
            TransformerFactory.newInstance();
        
        Transformer transformer =
            transformerFactory.newTransformer(new StreamSource(stylesXsltIS));
        
        transformer.transform(new StreamSource(src), new StreamResult(res));
     }
    
    public void testContentXslt() throws TransformerException
    {
        File src = new File("src/test/resources/com/galactic/odt/FeeTransaction.xml");
        File res = new File(this.outputDir,"content.xml");
       
        InputStream contentXsltIS =
            TestTransform.class.getClassLoader().getResourceAsStream("org/clazzes/odtransform/odtcontent.xslt");
        
        TransformerFactory transformerFactory =
            TransformerFactory.newInstance();
        
        Transformer transformer =
            transformerFactory.newTransformer(new StreamSource(contentXsltIS));
        
        transformer.transform(new StreamSource(src), new StreamResult(res));
     }
    
    public void testWriter() throws IOException, TransformerConfigurationException, SAXException
    {
        ZipFile odtFile = new ZipFile("src/test/resources/com/galactic/odt/FeeTransaction.odt");
        
        File res = new File(this.outputDir,"FeeTransaction.odt");
        File src = new File("src/test/resources/com/galactic/odt/FeeTransaction.xml");

        FileOutputStream ros = new FileOutputStream(res);
        ZipOutputStream zos = new ZipOutputStream(ros);
        
        OdtContentWriter writer = new OdtContentWriter(odtFile,zos);
        
        SAXTransformerFactory transformerFactory =
            (SAXTransformerFactory) SAXTransformerFactory.newInstance();
        
        TransformerHandler transformerHandler = transformerFactory.newTransformerHandler();
        
        transformerHandler.setResult(new SAXResult(writer));
        
        XMLReader xmlReader = XMLReaderFactory.createXMLReader();
        
        xmlReader.setContentHandler(transformerHandler);
        
        xmlReader.parse(new InputSource(new FileInputStream(src)));
        
        zos.close();

        this.testMergeInternal(res.getPath(), "FeeTransaction2.xml");
    }
}
